import 'dart:html' as html;

import 'package:dart_webrtc/dart_webrtc.dart';
import 'package:videosdk/src/core/room/audio_html/audio_html_interface.dart';
// ignore: implementation_imports
import 'package:dart_webrtc/src/media_stream_track_impl.dart';
import 'package:videosdk/src/core/webrtc/webrtc_client.dart';

class AudioHTMLWeb implements AudioHTMLInterface {
  final Map<String, html.Element> _audioElements = {};

  @override
  Future<dynamic> startAudio(Consumer? audioConsumer) async {
    if (audioConsumer == null || audioConsumer!.track == null) {
      return;
    }

    MediaStreamTrack track = audioConsumer.track;

    if (track is! MediaStreamTrackWeb) {
      return;
    }

    final elementId = "audio_${track.id}";
    var audioElement = html.document.getElementById(elementId);
    if (audioElement == null) {
      audioElement = html.AudioElement()
        ..id = elementId
        ..autoplay = true;
      findOrCreateAudioContainer().append(audioElement);
      _audioElements[track.id!] = audioElement;
    }

    if (audioElement is! html.AudioElement) {
      return;
    }
    final audioStream = html.MediaStream();
    audioStream.addTrack(track.jsTrack);
    audioElement.srcObject = audioStream;
    return audioElement.play();
  }

  html.DivElement findOrCreateAudioContainer() {
    var div = html.document.getElementById("videosdk_audio_container");
    if (div != null) {
      return div as html.DivElement;
    }

    div = html.DivElement();
    div.id = "videosdk_audio_container";
    div.style.display = 'none';
    html.document.body?.append(div);
    return div as html.DivElement;
  }

  @override
  void stopAudio(String id) {
    final audioElement = html.document.getElementById("audio_$id");
    if (audioElement != null) {
      if (audioElement is html.AudioElement) {
        audioElement.srcObject = null;
      }
      _audioElements.remove(id);
      audioElement.remove();
    }
  }
}

AudioHTMLInterface getInterface() => AudioHTMLWeb();
